<?php
/**
 * @file Admin UI for image ownage settings
 */

/**
* Admin settings page.
*/
function image_ownage_admin_settings() {
  $settings = variable_get('image_ownage_settings', image_ownage_default_settings());
  $form = array('image_ownage_settings' => image_ownage_settings_form($settings));
  $form['#tree'] = TRUE;
  return system_settings_form($form);
}

/**
 * Subform that can be used to set the image_ownage preferences.
 *
 * Normally this is called as a straight system_settings_form, but it may also be used as a subsettings config page when using rules, if you need to make rules-based processes that have different, um, rules.
 *
 * @param array $settings
 */
function image_ownage_settings_form(array $settings) {
  $form = array();

  $form['import_nearby'] = array(
    '#type' => 'checkbox',
    '#title' => t('When an embedded image is found that is local, but not in our expected directory, copy it to the correct files directory'),
    '#default_value' => $settings['import_nearby'],
  );
  $form['node_save'] = array(
    '#type' => 'checkbox',
    '#title' => t('Automatically process for embedded images on every node save.'),
    '#default_value' => $settings['node_save'],
  );
  $form['show_tab'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show an image embed action tab on each node. (may need a menu rebuild to take effect)'),
    '#default_value' => $settings['show_tab'],
  );

  $form['path'] = array(
    '#type' => 'textfield',
    '#title' => t('Where found files should all be kept (inside the files directory %file_directory_path). Leaving it blank is fine.', array('%file_directory_path' => file_directory_path())),
    '#default_value' => $settings['path'],
    '#description' => t('This value may be overridden if using filefield or a module which changes your storage paths. This path is only used if using the simpler attachment options below.'),
  );

  // Declare this element placeholder first, so we can add things underneath during the iteration.
  $form['attach_method'] = array();

  $attachment_methods = image_ownage_attachment_methods();

  foreach ($attachment_methods as $method => $details) {
    $options[$method] = $details['label'];
    // If the method defines the need for further settings,
    // add those subsettings from the form callback.
    if (isset($details['subform']) && function_exists($details['subform'])) {
      $subform_func = $details['subform'];
      $subform = $subform_func($settings[$method]);
    }
    else {
      $subform = array();
    }
    $form[$method] = array(
      '#type' => 'fieldset',
      '#title' => $details['label'],
      '#attributes' => array('class' => $method . ' image-attachment-method-options'),
      '#description' => $details['description'],
    ) + $subform;
  }

  $form['attach_method'] = array(
    '#title' => "How to attach found images to the containing node",
    '#type' => 'select',
    '#options' => $options,
    '#description' => $description,
    '#default_value' => $settings['attach_method'],
    '#element_validate' => array('image_ownage_attach_method_validate'),
  );

  ///////////////////
  // File ownage also
  $form['file_ownage'] = array(
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#title' => 'Linked files',
    '#description' => t('
      Also try to ensure linked files -
      <code>&lt;a href="path/to/doc.pdf"&gt;pdf file&lt;/a&gt; ,
      &lt;a href="path/to/fullsize.jpg"&gt;large image&lt;/a&gt;</code>.
      If found, the contents of links like this should be absorbed and
      owned also.
      ', array()),
    '#tree' => TRUE,
  );
  $form['file_ownage']['import_files'] = array(
    '#type' => 'checkbox',
    '#title' => t('Try to absorb linked files.'),
    '#default_value' => $settings['file_ownage']['import_files'],
  );
  $form['file_ownage']['suffix_list'] = array(
    '#type' => 'textfield',
    '#title' => t('List of suffixes to treat as file resources'),
    '#default_value' => $settings['file_ownage']['suffix_list'],
    '#description' => t('
      As we can\'t tell what file type is at the end of a link request,
      guess based on apparent suffix.
      Enter a comma-separated list, eg <code>pdf,doc,jpg</code>
      Be aware that this can cause direct links to remotely hosted resources
      to get copied locally.
      '),
  );
  // Add this placeholder here so it appears above the subform options
  $form['file_ownage']['attach_method'] = array();

  $attachment_methods = image_ownage_file_attachment_methods();
  $options = array();
  foreach ($attachment_methods as $method => $details) {
    $options[$method] = $details['label'];
    // If the method defines the need for further settings,
    // add those subsettings from the form callback.
    if (isset($details['subform']) && function_exists($details['subform'])) {
      $subform_func = $details['subform'];
      $subform = $subform_func($settings[$method]);
    }
    else {
      $subform = array();
    }
    $form['file_ownage'][$method] = array(
      '#type' => 'fieldset',
      '#title' => $details['label'],
      '#attributes' => array('class' => $method . ' file-attachment-method-options'),
      '#description' => $details['description'],
    ) + $subform;
  }

  $form['file_ownage']['attach_method'] = array(
    '#title' => "How to attach found images to the containing node",
    '#type' => 'select',
    '#options' => $options,
    '#description' => $description,
    '#default_value' => $settings['file_ownage']['attach_method'],
    '#element_validate' => array('image_ownage_attach_method_validate'),
  );




  // Additional options

  $strip_paths = array_filter((array) $settings['strip_paths']);
  $strip_paths[] = '';
  $form['strip_paths'] = array(
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#title' => 'Filepath cleanup',
    '#description' => t('You can provide a regular expression that could match part of a filepath to strip when calculating the new import file storage path. EG, if the target has paths like <code>_data/assets/image</code>', array()),
    '#tree' => TRUE,
  );

  foreach ($strip_paths as $pattern) {
    $form['strip_paths'][] = array(
      '#type' => 'textfield',
      '#default_value' => $pattern,
    );
  }

  $form['domains'] = array(
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#title' => 'Domain management',
  );
  $form['domains']['domain_handler'] = array(
    '#type' => 'radios',
    '#title' => t('Restrict source domains'),
    '#options' => array(
      'ownage' => 'Own everything we can get. If references to external images are found, they will get localized',
      'exclude' => 'Do not own images from the listed domains',
      'include' => 'Own images only from these domains',
    ),
    '#default_value' => $settings['domains']['domain_handler'],
  );
  $form['domains']['domain_list'] = array(
    '#type' => 'textarea',
    '#title' => t('Importing from external domains'),
    '#default_value' => $settings['domains']['domain_list'],
    '#description' => t('Add one URL per line. Start with http:// and feel ' .
      'free to include paths'),
  );

  $form['lost_files'] = array(
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#title' => 'Finding lost files',
  );
  $form['lost_files']['help'] = array(
    '#type' => 'markup',
    '#value' => '<p>
      If the linked file cannot be found where expected,
      we can try looking in alternative locations. This could help
      with importing old content, or just fixing up broken links.
      </p><p>
      Enter some paths to scan to see if a matching file is found there.
      This can even be a remote URL.
      Paths will be searched in order, first match being the best.
      </p><p>
      To <em>search</em> for files, enter a path with a wildcard <b>*</b>
      and that location will be scanned for the best match to the expected
      filename. Wildcard searches will not work on URLs.
      If a match is found, the processing rules above will be used and the
      source link will be updated to the correct location.
      </p>
    ',
  );
  $form['lost_files']['search_paths'] = array(
    '#type' => 'textarea',
    '#size' => 4,
    '#default_value' => $settings['lost_files']['search_paths'],
  );

  $form['loglevel'] = array(
    '#title' => t('Log level'),
    '#description' => t('For debugging, messages at this level or higher will be echoed to the screen. "notice" is probably the most informative as it describes changes that get made during the process.'),
    '#type' => 'select',
    '#options' => watchdog_severity_levels(),
    '#default_value' => $settings['loglevel'],
  );

  drupal_add_js(drupal_get_path('module', 'image_ownage') . '/image_ownage.js');
  return $form;
}

/**
 * Tidy up some values.
 *
 * @param unknown_type $form
 * @param unknown_type $form_state
 */
function image_ownage_admin_settings_validate(&$form, &$form_state) {
  $form_state['values']['image_ownage_settings']['lost_files']['search_paths'] = trim($form_state['values']['image_ownage_settings']['lost_files']['search_paths']);
}

function image_ownage_attach_method_validate(&$element, &$form_state) {
  // Ensure some of the options have the required dependencies
  switch ($element['#value']) {

    case 'image_ownage_attach_file' :
      if (! module_exists('upload')) {
        form_error($element, t('This method requires the upload module to be enabled.'));
      }
    break;

    case 'image_ownage_attach_filefield' :
      if (! module_exists('filefield')) {
        form_error($element, t('This method requires the filefield module to be enabled.'));
      }
    break;

    case 'image_ownage_attach_image' :
      if (! module_exists('image_attach')) {
        form_error($element, t('This method requires the image_attach module to be enabled.'));
      }
    break;

    case 'image_ownage_attach_imageholder_nodereference' :
      if (! module_exists('nodereference')) {
        form_error($element, t('This method requires the nodereference module to be enabled.'));
      }
    break;

    case 'image_ownage_attach_imageholder_wysiwyg' :
      if (! module_exists('wysiwyg_imageupload')) {
        form_error($element, t('This method requires the wysiwyg_imageupload module to be enabled.'));
      }
    break;

  }

}

/**
 * Extra preferences to support the filefield option.
 */
function image_ownage_attach_filefield_subform($subsettings) {
  $available_filefields = image_ownage_available_fields('filefield');

  $subform = array();
  $subform['about_filefields'] = array(
    '#type' => 'markup',
    '#value' => '<p>' . t('The filefields you select should be consistent and re-used throughout your <a href="!content_types_url">content types</a>. If a filefield is unavailable for a node, it will not be used.', array('!content_types_url' => url('admin/content/types'))) . '</p>',
  );
  $subform['filefield_name'] = array(
    '#title' => "Which filefield to use for supplimentary images",
    '#type' => 'select',
    '#options' => $available_filefields,
    '#default_value' => $subsettings['filefield_name'],
    '#description' => 'This filefield is expected to have "unlimited" entries.',
  );
  return $subform;
}


/**
 * Extra preferences to support the feature filefield option.
 */
function image_ownage_attach_feature_filefield_subform($subsettings) {
  $available_filefields = image_ownage_available_fields('filefield');

  $subform = array();
  $subform['about_filefields'] = array(
    '#type' => 'markup',
    '#value' => '<p>' . t('The filefields you select should be consistent and re-used throughout your <a href="!content_types_url">content types</a>. If a filefield is unavailable for a node, it will not be used.', array('!content_types_url' => url('admin/content/types'))) . '</p>',
  );
  $subform['feature_filefield_name'] = array(
    '#title' => "Which filefield to use for feature image",
    '#type' => 'select',
    '#options' => $available_filefields,
    '#default_value' => $subsettings['feature_filefield_name'],
    '#description' => 'This option supports attaching one image that may be rendered uniquely (eg as a representative teaser image) while the others go inline. This filefield chosen here is expected to have a limit of one item.',
  );
  $subform['feature_filefield_min_width'] = array(
    '#title' => "What is the minimum width before an embedded image is eligible to become a featured image.",
    '#type' => 'textfield',
    '#size' => 4,
    '#default_value' => $subsettings['feature_filefield_min_width'],
    '#description' => 'You don\'t want small images accidentally being chosen to be the feature image. Enter a minimum width in pixels. Images smaller than that will not become features.',
  );
  $subform['remove_original'] = array(
    '#type' => 'checkbox',
    '#title' => t('When an embedded feature image is found and attached, remove it from the normal text flow'),
    '#default_value' => $settings['remove_original'],
    '#description' => t('This allows you to reformat the feature image and theme it your own way. It only applies on "feature" iamges, not supplimentary ones'),
  );

  return $subform;
}

/**
 * Extra preferences to support the imageholder_nodereference option.
 */
function image_ownage_attach_imageholder_nodereference_subform($subsettings) {
  $available_content_types = node_get_types('names');
  $available_nodereferencefields = image_ownage_available_fields('nodereference');
  $available_filefields = image_ownage_available_fields('filefield');

  $subform = array();
  $subform['imageholder_type'] = array(
    '#title' => "This content type to create as a placeholder for new images",
    '#type' => 'select',
    '#options' => $available_content_types,
    '#default_value' => $subsettings['imageholder_type'],
    '#description' => 'You may have to make a new content type called, eg, "image" and add an imagefield to it.',
  );
  $subform['filefield_name'] = array(
    '#title' => "Which filefield on the target content type to store the image in",
    '#type' => 'select',
    '#options' => $available_filefields,
    '#default_value' => $subsettings['filefield_name'],
  );
  $subform['nodereference_field'] = array(
    '#title' => "The nodereference field used to link from the parent node content to the image holder.",
    '#type' => 'select',
    '#options' => $available_nodereferencefields,
    '#default_value' => $subsettings['nodereference_field'],
  );
  return $subform;
}

/**
 * Extra preferences to support the imageholder_wysiwyg option.
 */
function image_ownage_attach_imageholder_wysiwyg_subform($subsettings) {
  $available_content_types = node_get_types('names');
  $available_filefields = image_ownage_available_fields('filefield');

  $subform = array();
  $subform['imageholder_type'] = array(
    '#title' => "This content type to create as a placeholder for new images",
    '#type' => 'select',
    '#options' => $available_content_types,
    '#default_value' => $subsettings['imageholder_type'],
    '#description' => 'You may have to make a new content type called, eg, "image" and add an imagefield to it.',
  );
  $subform['filefield_name'] = array(
    '#title' => "Which filefield on the target content type to store the image in",
    '#type' => 'select',
    '#options' => $available_filefields,
    '#default_value' => $subsettings['filefield_name'],
  );
  // additional wysiwyg_embed options?
  return $subform;
}

/**
 * Return an array of all instances of filefield from any content type.
 *
 * Lookup utility for the selection form.
 * @param field type, eg 'filefield', 'nodereference'
 */
function image_ownage_available_fields($field_type) {
  $available_fields = array();
  if (function_exists('content_types')) {
    foreach (content_types() as $content_def) {
      foreach ($content_def['fields'] as $field_id => $field_def) {
        if ($field_def['type'] == $field_type) {
          $available_fields[$field_id] = $field_def['field_name'];
        }
      }
    }
  }
  return $available_fields;
}

