<?php

/**
 * @file RULES based processing for processing embedded items.
 *
 * There are so many branches and ways this logic could go,
 * I'll expose the logic through rules.
 */

// hook_rules_action_info
// hook_rules_condition_info
// hook_rules_data_type_info *
// hook_rules_event_info


/**
 * Publish the availablity of an event that fires when a suitable domelement element is found.
 *
 * Implementation of the rules hook_rules_event_info
 * Docs at: http://drupal.org/node/298549
 */
function image_ownage_rules_event_info() {
  $events = array();
  $events['embed_element_found'] = array(
    'label' => 'An embedded file link was found in the text',
    'module' => 'Image Ownage',
    'arguments' => array(
      'element' => array(
        'type' => 'domelement',
        'label' => t('Found DOM element'),
      ),
      'node' => array(
        'type' => 'node',
        'label' => t('Node context'),
      ),
    ),
  );
  return $events;
}

/**
 * Implementation of hook_hook_info().
 */
function image_ownage_hook_info() {
  return array(
    'embed' => array(
      'embed' => array(
        'update' => array(
          'runs when' => t('An embedded link is found in the text'),
        ),
      ),
    ),
  );
}


/**
 * Implimentation of hook_action_info().
 */
function image_ownage_action_info() {
  return array(
    'image_ownage_scan_embeds_action' => array(
      'type' => 'node',
      'description' => t('Scan the page body for embedded elements'),
      'configurable' => TRUE,
      'hooks' => array('any' => array(TRUE)),
      // The expected input to this action
      'arguments' => array(
        'node' => array(
          'type' => 'node',
          'label' => t('Content'),
        ),
      ),
      // If this processes right, the result will be a 'new variable' in the form of an element we can then process
      'new variables' => array(
        'element' => array(
          'type' => 'domelement',
          'label' => t('Found DOM element'),
        ),
      ),
    ),
    'image_ownage_add_as_upload_action' => array(
      'type' => 'domelement',
      'description' => t('Adds the found file as an upload attachment to the owner node'),
      'configurable' => FALSE,
      'hooks' => array('any' => array(TRUE)),
    ),
  );
}

/**
 * Settings for the scan embeds process
 */
function image_ownage_scan_embeds_action_form() {
  dpm(func_get_args());

  $form = array();

  return $form;
}



/**
 * Announce the existence of a new data type - an object that we will take as an argument and manipulate.
 *
 * We deal with a new data type - a domelement element found during text processing
 */
function image_ownage_rules_data_type_info() {
  return array(
    'domelement' => array(
      'label' => t('HTML Element'),
      'class' => 'rules_data_type_domelement',
      'savable' => FALSE,
      'identifiable' => FALSE,
      'module' => 'Embed',
    ),
  );
}

/**
 * Defines the domelament type.
 *
 * This publishes the interface to things we are allowed to do to a domelement object.
 */
class rules_data_type_domelement extends rules_data_type {
  // I dunno what goes here yet
}
